import gdal
import numpy as np
import os
from cogconverter.src.pyramid import pyramid

# Remappping array to
def remap_array(arr):
    """
    Remapping [4, 256,256] to [256,256, 4]
    """
    return np.moveaxis(arr, 0, 2)


def create_alpha(raster):
    r = raster
    blocksize = 256
    # Creating a copy in mem
    path_alpha = os.path.join(os.path.dirname(r.path_input), 'alpha.tif')

    # Creating a copy in mem

    vrt_ds = gdal.GetDriverByName('mem').CreateCopy('', r.ds, 0)

    print('Number of bands : %s' % (vrt_ds.RasterCount))
    vrt_ds.AddBand()
    vrt_ds.GetRasterBand(4).SetColorInterpretation(gdal.GCI_AlphaBand)

    print('Processing: Creating alpha tiff dataset')

    driver = gdal.GetDriverByName('Gtiff')
    dataset = driver.CreateCopy(path_alpha,
                                vrt_ds, 0,
                                ['NUM_THREADS=ALL_CPUS',
                                 'COMPRESS=%s' % (r.compression),
                                 'BIGTIFF=YES',
                                 'TILED=YES',
                                 'BLOCKXSIZE=%d' % (blocksize),
                                 'BLOCKYSIZE=%d' % (blocksize),
                                 'COPY_SRC_OVERVIEWS=YES'])

    vrt_ds = None

    output_band = dataset.GetRasterBand(4)

    # block_x, block_y = input_band.GetBlockSize()
    block_x, block_y = 256, 256

    size_x = r.ds.RasterXSize
    size_y = r.ds.RasterYSize

    for x in range(0, int(size_x), int(block_x)):
        if x + block_x < size_x:
            col = block_x
        else:
            col = size_x - x

        for y in range(0, int(size_y), int(block_y)):
            if y + block_y < size_y:
                row = block_y
            else:
                row = size_y - y

            array = r.ds.ReadAsArray(x, y, col, row)
            all_zeros = array != 0
            zeros_image = remap_array(all_zeros)
            mask = np.all(zeros_image, axis=2) * 255
            output_band.WriteArray(mask, x, y)

    print('Added extra band, Number of bands : %s' % (dataset.RasterCount))

    # Creating alpha pyramids
    print('Generating alpha Overviews')
    addo = pyramid(dataset)
    addo.gdal_addo()
    
    print('Saving to Disk')
    dataset.FlushCache()
    dataset = None
    return path_alpha



import logging
from logging.handlers import TimedRotatingFileHandler
import os
import sys


# Multiple calls to logging.getLogger('someLogger') return a
# reference to the same logger object.  This is true not only
# within the same module, but also across modules as long as
# it is in the same Python interpreter process.

FORMATTER = logging.Formatter(
    "%(asctime)s — %(name)s — %(levelname)s —"
    "%(funcName)s:%(lineno)d — %(message)s")


def get_console_handler():
    console_handler = logging.StreamHandler(sys.stdout)
    console_handler.setFormatter(FORMATTER)
    return console_handler



import gdal
import osgeo
from cogconverter.config import default_config


# Creating pyramids/overviews
class pyramid(object):
    def __init__(self, dataset):
        self.dataset = dataset
        assert isinstance(dataset, osgeo.gdal.Dataset), __name__ + 'Excepted osgeo.gdal class type'


    def gdal_addo(self):
        # Setting no data value
        for i in range(self.dataset.RasterCount):
            band = self.dataset.GetRasterBand(i+1)
            band.SetNoDataValue(default_config.NO_DATA)

        # 0 = read-only, 1 = read-write
        overviews = self.dataset.GetRasterBand(1).GetOverviewCount()
        if overviews == 0:
            gdal.SetConfigOption('COMPRESS_OVERVIEW', default_config.COMPRESS)
            self.dataset.BuildOverviews(default_config.RESAMPLING, [2, 4, 8, 16, 32, 64])
            print('Completed: Generating overviews')
        else:
            print('Overviews already generated. Thus skipping!')



from osgeo import gdal
import argparse

import os
import sys

import osr
import json
import time
import datetime


class ValidateCloudOptimizedGeoTIFFException(Exception):
    pass


def validate(ds, check_tiled=True):
    """Check if a file is a (Geo)TIFF with cloud optimized compatible structure.

    Args:
      ds: GDAL Dataset for the file to inspect.
      check_tiled: Set to False to ignore missing tiling.

    Returns:
      A tuple, whose first element is an array of error messages
      (empty if there is no error), and the second element, a dictionary
      with the structure of the GeoTIFF file.

    Raises:
      ValidateCloudOptimizedGeoTIFFException: Unable to open the file or the
        file is not a Tiff.
    """

    if int(gdal.VersionInfo('VERSION_NUM')) < 2020000:
        raise ValidateCloudOptimizedGeoTIFFException(
            'GDAL 2.2 or above required')

    unicode_type = type(''.encode('utf-8').decode('utf-8'))
    if isinstance(ds, (str, unicode_type)):
        gdal.PushErrorHandler()
        ds = gdal.Open(ds)
        gdal.PopErrorHandler()
        if ds is None:
            raise ValidateCloudOptimizedGeoTIFFException(
                'Invalid file : %s' % gdal.GetLastErrorMsg())
        if ds.GetDriver().ShortName != 'GTiff':
            raise ValidateCloudOptimizedGeoTIFFException(
                'The file is not a GeoTIFF')

    details = {}
    errors = []
    warnings = []
    filename = ds.GetDescription()
    main_band = ds.GetRasterBand(1)
    ovr_count = main_band.GetOverviewCount()
    filelist = ds.GetFileList()
    if filelist is not None and filename + '.ovr' in filelist:
        errors += [
            'Overviews found in external .ovr file. They should be internal']

    if main_band.XSize >= 512 or main_band.YSize >= 512:
        if check_tiled:
            block_size = main_band.GetBlockSize()
            if block_size[0] == main_band.XSize and block_size[0] > 1024:
                errors += [
                    'The file is greater than 512xH or Wx512, but is not tiled']

        if ovr_count == 0:
            warnings += [
                'The file is greater than 512xH or Wx512, it is recommended '
                'to include internal overviews']

    ifd_offset = int(main_band.GetMetadataItem('IFD_OFFSET', 'TIFF'))
    ifd_offsets = [ifd_offset]
    if ifd_offset not in (8, 16):
        errors += [
            'The offset of the main IFD should be 8 for ClassicTIFF '
            'or 16 for BigTIFF. It is %d instead' % ifd_offsets[0]]
    details['ifd_offsets'] = {}
    details['ifd_offsets']['main'] = ifd_offset

    for i in range(ovr_count):
        # Check that overviews are by descending sizes
        ovr_band = ds.GetRasterBand(1).GetOverview(i)
        if i == 0:
            if (ovr_band.XSize > main_band.XSize or
                    ovr_band.YSize > main_band.YSize):
                errors += [
                    'First overview has larger dimension than main band']
        else:
            prev_ovr_band = ds.GetRasterBand(1).GetOverview(i - 1)
            if (ovr_band.XSize > prev_ovr_band.XSize or
                    ovr_band.YSize > prev_ovr_band.YSize):
                errors += [
                    'Overview of index %d has larger dimension than '
                    'overview of index %d' % (i, i - 1)]

        if check_tiled:
            block_size = ovr_band.GetBlockSize()
            if block_size[0] == ovr_band.XSize and block_size[0] > 1024:
                errors += [
                    'Overview of index %d is not tiled' % i]

        # Check that the IFD of descending overviews are sorted by increasing
        # offsets
        ifd_offset = int(ovr_band.GetMetadataItem('IFD_OFFSET', 'TIFF'))
        ifd_offsets.append(ifd_offset)
        details['ifd_offsets']['overview_%d' % i] = ifd_offset
        if ifd_offsets[-1] < ifd_offsets[-2]:
            if i == 0:
                errors += [
                    'The offset of the IFD for overview of index %d is %d, '
                    'whereas it should be greater than the one of the main '
                    'image, which is at byte %d' %
                    (i, ifd_offsets[-1], ifd_offsets[-2])]
            else:
                errors += [
                    'The offset of the IFD for overview of index %d is %d, '
                    'whereas it should be greater than the one of index %d, '
                    'which is at byte %d' %
                    (i, ifd_offsets[-1], i - 1, ifd_offsets[-2])]

    # Check that the imagery starts by the smallest overview and ends with
    # the main resolution dataset
    block_offset = main_band.GetMetadataItem('BLOCK_OFFSET_0_0', 'TIFF')
    if not block_offset:
        errors += ['Missing BLOCK_OFFSET_0_0']
    data_offset = int(block_offset) if block_offset else None
    data_offsets = [data_offset]
    details['data_offsets'] = {}
    details['data_offsets']['main'] = data_offset
    for i in range(ovr_count):
        ovr_band = ds.GetRasterBand(1).GetOverview(i)
        data_offset = int(ovr_band.GetMetadataItem('BLOCK_OFFSET_0_0', 'TIFF'))
        data_offsets.append(data_offset)
        details['data_offsets']['overview_%d' % i] = data_offset

    if data_offsets[-1] < ifd_offsets[-1]:
        if ovr_count > 0:
            errors += [
                'The offset of the first block of the smallest overview '
                'should be after its IFD']
        else:
            errors += [
                'The offset of the first block of the image should '
                'be after its IFD']
    for i in range(len(data_offsets) - 2, 0, -1):
        if data_offsets[i] < data_offsets[i + 1]:
            errors += [
                'The offset of the first block of overview of index %d should '
                'be after the one of the overview of index %d' %
                (i - 1, i)]
    if len(data_offsets) >= 2 and data_offsets[0] < data_offsets[1]:
        errors += [
            'The offset of the first block of the main resolution image'
            'should be after the one of the overview of index %d' %
            (ovr_count - 1)]

    return warnings, errors, details


"""
-co TILED=YES -co COMPRESS=JPEG -co PHOTOMETRIC=YCBCR -co COPY_SRC_OVERVIEWS=YES \
-co BLOCKXSIZE=512 -co BLOCKYSIZE=512 --config GDAL_TIFF_OVR_BLOCKSIZE 512
"""


def main():
    """Return 0 in case of success, 1 for failure."""

    filename = path_input

    if filename is None:
        raise('Error: No input file is given. Exiting...')

    try:
        ret = 0
        warnings, errors, _ = validate(filename)
        if warnings:
            print('The following warnings were found:')
            for warning in warnings:
                print(' - ' + warning)

            print('Convert to cloud optimized GeoTIFF')

        elif errors:
            print('%s is NOT a valid cloud optimized GeoTIFF.' % filename)
            print('The following errors were found:')
            for error in errors:
                print(' - ' + error)

            print('Convert to cloud optimized GeoTIFF')
        else:
            print('%s is a valid cloud optimized GeoTIFF' % filename)

    except ValidateCloudOptimizedGeoTIFFException as e:
        print('%s is NOT a valid cloud optimized GeoTIFF : %s' %
              (filename, str(e)))

        print('Convert to cloud optimized GeoTIFF')

    return ret


if __name__ == '__main__':

    parser = argparse.ArgumentParser()

    parser.add_argument('-p', '--payload',
                        help='Pass input file', required=True)

    args = parser.parse_args()
    path_input = args.payload

    if not os.path.exists(path_input):
        raise('Error: File not Found')
    
    # Validator starts here
    try:
        main()
    except Exception as e:
        raise('Error: Unable to validate and convert. %s' % e)
    sys.exit()



import os
import osr
import json
import utm
import gdal
import datetime


# Metadata class
class Metadata(object):

    def __init__(self, payload):
        self.payload = payload

    # Check directory
    def checkdirs(self, output_folder):
        if not os.path.exists(output_folder):
            os.makedirs(os.path.relpath(output_folder))

    # saving to JSON
    def tojson(self, dictA, file_json):
        with open(file_json, 'w') as f:
            json.dump(dictA, f, indent=4, separators=(',', ': '))

    # EPSG Code to Zone number
    def code2zone(self, epsg_code):
        epsg_code = int(epsg_code)
        last = int(repr(epsg_code)[-1])
        sec_last = int(repr(epsg_code)[-2])
        zone = sec_last*10 + last
        return zone

    # Get EPSG code
    def get_code(self, dataset):
        proj = osr.SpatialReference(wkt=dataset.GetProjection())
        return proj.GetAttrValue('AUTHORITY', 1)

    # Get Bounding Box
    def GetExtent(self, ds, row, col):
        gt = ds.GetGeoTransform()

        originX = gt[0]
        originY = gt[3]
        px_width = gt[1]
        px_height = gt[5]

        epsg = self.get_code(ds)

        max_x = originX + col*px_width
        max_y = originY
        min_x = originX

        if px_height < 0:
            # Since value of height is negative
            min_y = originY + row*px_height

        if px_height > 0:
            # Since value of height is negative
            min_y = originY - row*px_height

    #    if int(epsg) != 4326:
    #        zone = code2zone(epsg)
    #        mini = utm.to_latlon(min_x, min_y, zone, zone_letter='N')
    #        maxi = utm.to_latlon(max_x, max_y, zone, zone_letter='N')

    #    else:
        mini = [min_y, min_x]
        maxi = [max_y, max_x]

        return [mini[1], mini[0], maxi[1], maxi[0]]

    # Extracting metadata
    def extract(self):
        metadata = {}
        ds_geo = gdal.Warp('', self.payload, dstSRS='EPSG:4326', format='VRT')

        ds = gdal.Open(self.payload)

        num_band = ds.RasterCount

        # Checking array type
        banddata = ds.GetRasterBand(1)
        arr = banddata.ReadAsArray(0, 0, 1, 1)

        # Dimensions
        col = ds.RasterXSize
        row = ds.RasterYSize

        # Tranformation settings
        geotransform = ds.GetGeoTransform()

        # Getting Nodata values
        no_data = ds.GetRasterBand(1).GetNoDataValue()

        # Origin and pixel length
        originX = geotransform[0]
        originY = geotransform[3]
        px_width = geotransform[1]
        px_height = geotransform[5]

        # Generate bbox in latitude and longitude only
        bbox = self.GetExtent(ds_geo, row, col)

        # Getting time information
        try:
            import ntplib
            x = ntplib.NTPClient()
            timestamp = str(datetime.datetime.utcfromtimestamp(
                x.request('europe.pool.ntp.org').tx_time))
            time_source = 'internet'
        except:
            print('Could not sync with time server. Taking Local TimeStamp')
            timestamp = str(datetime.datetime.utcnow()).split('.')[0]
            time_source = 'local'

        # Generating metadata
        metadata['bbox'] = bbox
        metadata['numband'] = num_band
        metadata['epsgcode'] = self.get_code(ds)
        metadata['originx'] = originX
        metadata['originy'] = originY
        metadata['pixelwidth'] = px_width
        metadata['pixelheight'] = px_height
        metadata['size'] = [col, row]
        metadata['nodata'] = no_data
        metadata['datatype'] = str(arr[0][0].dtype)
        metadata['file'] = os.path.basename(self.payload)
        metadata['time'] = timestamp
        metadata['timesource'] = time_source

        return metadata


try:
    import osgeo
except:
    raise('osgeo not found. You can install using - conda install gdal. Else use docker file given at repo')
try:
    import gdal
except:
    raise('gdal not found. You can install using - conda install gdal. Else use docker file given at repo')

import numpy as np
import os
import sys
from tqdm import tqdm
import argparse
from cogconverter.config import default_config
from cogconverter.src import pyramid

"""
-co TILED=YES -co COMPRESS=JPEG -co PHOTOMETRIC=YCBCR -co COPY_SRC_OVERVIEWS=YES \
-co BLOCKXSIZE=512 -co BLOCKYSIZE=512 --config GDAL_TIFF_OVR_BLOCKSIZE 512
"""


class raster(object):

    def __init__(self, ds):
        self.intermediate_format = 'VRT'
        self.ds = ds
        self.config()

    def config(self):
        if self.ds is None:
            raise('Input file provided %s cannot be loaded' %
                  (self.ds))

        self.no_data = self.ds.GetRasterBand(1).GetNoDataValue()

        if self.no_data is None:
            self.no_data = default_config.NO_DATA
            # Reading tif dataset
            # ds = gdal.Open(file)

        # Reading compression type
        try:
            self.compression = self.ds.GetMetadata(
                'IMAGE_STRUCTURE')['COMPRESSION']
        except:
            self.compression = default_config.COMPRESS

        if self.compression.upper() == 'YCbCr JPEG'.upper():
            self.compression = 'JPEG'

        # Reading number of bands
        self.num_band = self.ds.RasterCount

        # Defining dtype
        self.dtype = self.ds.ReadAsArray(0, 0, 1, 1).dtype

        # Dimensions
        self.col = self.ds.RasterXSize
        self.row = self.ds.RasterYSize

        # Tranformation settings
        self.geotransform = self.ds.GetGeoTransform()

        # Projections
        self.geoprojection = self.ds.GetProjection()

        # Check for pyramids and overviews, 0 if no overviews
        self.overview = self.ds.GetRasterBand(1).GetOverviewCount()

        # # Origin and pixel length
        # self.originX = self.geotransform[0]
        # self.originY = self.geotransform[3]
        # self.px_width = self.geotransform[1]
        # self.px_height = self.geotransform[5]

        # # Generate bbox of original raster
        # self.bbox, self.origin = self.ds.GetExtent()

    def dtype_conversion(self):
        """
        GDT_Byte 	    8 bit unsigned integer
        GDT_UInt16 	    16 bit unsigned integer
        GDT_Int16 	    16 bit signed integer
        GDT_UInt32 	    32 bit unsigned integer
        GDT_Int32 	    32 bit signed integer
        GDT_Float32 	32 bit floating point
        GDT_Float64 	64 bit floating point
        """
        # Float 64
        if self.dtype == np.dtype(np.float64):
            output_dtype = gdal.GDT_Float64
            return output_dtype
        # Float 32
        elif self.dtype == np.dtype(np.float32):
            output_dtype = gdal.GDT_Float32
            return output_dtype
        # Uint 8
        elif self.dtype == np.dtype(np.uint8):
            output_dtype = gdal.GDT_Byte
            return output_dtype
        # Uint 16
        elif self.dtype == np.dtype(np.uint16):
            output_dtype = gdal.GDT_UInt16
            return output_dtype
        # Unit 32
        elif self.dtype == np.dtype(np.uint32):
            output_dtype = gdal.GDT_UInt32
            return output_dtype
        # Int 16
        elif self.dtype == np.dtype(np.int16):
            output_dtype = gdal.GDT_Int16
            return output_dtype
        # Int 32
        elif self.dtype == np.dtype(np.int32):
            output_dtype = gdal.GDT_Int32
            return output_dtype
        else:
            return gdal.GDT_Float32

    def gdal_addo(self):
        # 0 = read-only, 1 = read-write.
        if self.overview == 0:
            gdal.SetConfigOption('COMPRESS_OVERVIEW', 'LZW')
            self.ds.BuildOverviews("NEAREST", [2, 4, 8, 16, 32, 64])
            print('Completed: Generating overviews')
        else:
            print('Overviews already generated. Thus skipping!')

# Class over
######################################################################


def write_blockwise(input_raster, output_raster):

    assert isinstance(input_raster, raster), __name__ + \
        'Excepted raster class type'

    dimension = input_raster.num_band

    for num_band in range(dimension):
        print('Processing: Band %d' % (num_band))
        input_band = input_raster.ds.GetRasterBand(num_band+1)
        output_band = output_raster.GetRasterBand(num_band+1)
        block_x, block_y = input_band.GetBlockSize()
        # block_x, block_y = 5000, 5000

        size_x = input_band.XSize
        size_y = input_band.YSize

        for x in tqdm(range(0, int(size_x), int(block_x))):
            if x + block_x < size_x:
                col = block_x
            else:
                col = size_x - x

            for y in range(0, int(size_y), int(block_y)):
                if y + block_y < size_y:
                    row = block_y
                else:
                    row = size_y - y

                array = input_band.ReadAsArray(x, y, col, row)
                output_band.WriteArray(array, x, y)
                del array


def checkdirs(path):
    if not os.path.exists(path):
        os.makedirs(path)


def convert2blocksize(ds, path_output):
    '''
    path_input is input file name
    path_output is output file name
    '''
    assert isinstance(ds, osgeo.gdal.Dataset), __name__ + \
        'Excepted osgeo.gdal class type'

    blocksize = default_config.BLOCKSIZE

    r = raster(ds)
    # Loading cnfiguration
    r.config()

    # Checking for projection system
    if len(r.geoprojection) == 0:
        raise('Error: GeoProjection of input file is not defined')

    # Building overviews
    print('Processing: Building overviews')
    r.gdal_addo()

    # Creating tifs
    print('Processing: Creating tiff dataset')
    driver = gdal.GetDriverByName('Gtiff')
    try:
        dataset = driver.CreateCopy(path_output,
                                    r.ds, 0,
                                    ['NUM_THREADS=ALL_CPUS',
                                     'COMPRESS=%s' % (r.compression),
                                     'BIGTIFF=YES',
                                     'TILED=YES',
                                     'BLOCKXSIZE=%d' % (blocksize),
                                     'BLOCKYSIZE=%d' % (blocksize),
                                     'COPY_SRC_OVERVIEWS=YES'])
    except Exception as e:
        raise('Error: Unable to process %s' % e)

    print('Success: Creating tiff dataset completed')
    return dataset
    # driver = gdal.GetDriverByName('Gtiff')
    # dataset = driver.Create(path_output,
    #                         r.col, r.row, r.num_band,
    #                         r.dtype_conversion(), ['NUM_THREADS=ALL_CPUS',
    #                                                'COMPRESS=%s' % (
    #                                                    r.compression),
    #                                                'BIGTIFF=YES',
    #                                                'TILED=YES',
    #                                                'BLOCKXSIZE=%d' % (
    #                                                    blocksize),
    #                                                'BLOCKYSIZE=%d' % (
    #                                                    blocksize),
    #                                                'COPY_SRC_OVERVIEWS=YES'])
    # dataset.SetGeoTransform(r.geotransform)
    # dataset.SetProjection(r.geoprojection)

    # '''
    # No need to copy data again. Since we are using CreateCopy() function, it will automatically copy all the datasets
    # '''

    # # Copying data from input raster to output raster blockwise
    # write_blockwise(input_raster=r, output_raster=dataset)

    # '''
    # No Need to build overviews of output raster. It will remove COG properties from TIF
    # '''

    # # # Building overviews of output dataset, this will remove COG nature of TIF
    # print('Processing: Building overviews of output dataset')
    # addo = pyramid.pyramid(dataset)
    # addo.gdal_addo()
    # print('Success: Creating tiff dataset completed')
    # return dataset


if __name__ == '__main__':
    parser = argparse.ArgumentParser()

    parser.add_argument('-p', '--payload',
                        help='Pass input file',
                        required=True)

    parser.add_argument('-o', '--output',
                        help='Pass output file',
                        default=None,
                        required=False)

    args = parser.parse_args()
    path_input = args.payload
    path_output = args.output

    # Standard parameters
    coordinate = default_config.EPSG_CRS
    intermediate_format = default_config.INTERMEDIATE_FORMAT

    # Reading raster
    # ds = gdal.Open(path_input)
    if not os.path.exists(path_input):
        raise('Error: File not found')
    
    ds = gdal.Warp('', path_input, dstSRS=coordinate,
              format=intermediate_format)

    ds1 = convert2blocksize(ds, path_output)
    ds = None
    try:
        print('Processing: Flushing')
        ds1.FlushCache()
        ds1 = None
        print('Success: Process Completed')
    except Exception as e:
        raise('Error: Unable to save to %s %s' % (path_output, e))

    sys.exit()



import logging
import os

from cogconverter.config import default_config
from cogconverter.config import logging_config
import cogconverter.validator
import  cogconverter.converter


# Configure logger for use in package
logger = logging.getLogger(__name__)
logger.setLevel(logging.DEBUG)
logger.addHandler(logging_config.get_console_handler())
logger.propagate = False


import re
from io import open

from setuptools import find_packages, setup

# Get the long description from the relevant file
with open('README.rst', encoding='utf-8') as f:
    long_description = f.read()

with open('cogconverter/VERSION') as version_file:
    __version__=version_file.read().strip()

setup(name='cogconverter',
      version=__version__,
      description="Utility to convert raster dataset to Cloud Optimized GeoTIFFs",
      long_description=long_description,
      classifiers=[
          'License :: OSI Approved :: GNU General Public License v3 or later (GPLv3+)',
          'Programming Language :: Python :: 3.6',
          'Programming Language :: Python :: 3.7',
          'Topic :: Scientific/Engineering :: GIS',
          'Topic :: Utilities',
      ],
      keywords='GIS, Cloud, GDAL',
      author="Manish Sahu",
      author_email='manish.sahu.civ13@iitbhu.ac.in',
      url='https://github.com/ManishSahu53/cogconverter.git',
      license='GPLv3+',
      packages=find_packages(exclude=['ez_setup', 'examples', 'tests']),
      include_package_data=True,
      zip_safe=False,
      install_requires=open('requirements.txt').read().splitlines(),
      extras_require={
          'dev': [
              'numpy',
              'argparse',
              'tqdm'
          ],
      },
    #   entry_points="""
    #   [console_scripts]
    #   sentinelsat=sentinelsat.scripts.cli:cli
    #   """
)

# MD to  RST
# pandoc --from=markdown --to=rst --output=README.rst README.md